/* -*- mode: c; c-basic-offset: 4; -*- */
/* vi:set cindent ts=8 sts=4 sw=4: */
#include <assert.h>
#include <ncurses.h>
#include <stdio.h>
#include <stdlib.h>

#include "main.h"

#include "curs_color.h"
#include "options.h"

/*
 * Default color definitions. Change these if you want different colors.
 * If you're adding or removing some of these, don't forget curs_color.h
 * FG = foreground (the text printed)
 * BG = background
 * HL = highlight (if the text printed should be bold)
 * NOCOLOR_ATTR (attributes to use when you don't use colors)
 */
/* The status line */
#define STATUS_FG                     COLOR_CYAN
#define STATUS_BG                     COLOR_BLUE
#define STATUS_HL                     FALSE
#define STATUS_NOCOLOR_ATTR           A_REVERSE

/* The highlighted menu entry */ 
#define CURRENT_ENTRY_BG             COLOR_WHITE
#define CURRENT_ENTRY_FG             COLOR_BLACK
#define CURRENT_ENTRY_HL             FALSE
#define CURRENT_ENTRY_NOCOLOR_ATTR   A_REVERSE

/* The command reference */
#define CMD_REF_FG                   COLOR_CYAN
#define CMD_REF_BG                   COLOR_BLUE
#define CMD_REF_HL                   FALSE
#define CMD_REF_NOCOLOR_ATTR         A_REVERSE

/* The message line */
#define MSG_LINE_FG                  COLOR_WHITE
#define MSG_LINE_BG                  COLOR_BLACK
#define MSG_LINE_HL                  FALSE
#define MSG_LINE_NOCOLOR_ATTR        A_NORMAL

/* The rest of the screen */
#define MAIN_FG                      COLOR_WHITE
#define MAIN_BG                      COLOR_BLACK
#define MAIN_HL                      FALSE
#define MAIN_NOCOLOR_ATTR            A_NORMAL

/* Table of color values, the order must correspond
   with the macros in curs_color.h */
static int color_table[][5] =
{
    {STATUS_FG, STATUS_BG,
     STATUS_HL, STATUS_NOCOLOR_ATTR},
    {CURRENT_ENTRY_FG, CURRENT_ENTRY_BG,
     CURRENT_ENTRY_HL, CURRENT_ENTRY_NOCOLOR_ATTR},
    {CMD_REF_FG, CMD_REF_BG,
     CMD_REF_HL, CMD_REF_NOCOLOR_ATTR},
    {MSG_LINE_FG, MSG_LINE_BG,
     MSG_LINE_HL, MSG_LINE_NOCOLOR_ATTR},
    {MAIN_FG, MAIN_BG,
     MAIN_HL, MAIN_NOCOLOR_ATTR},
};

chtype ColorAttributes[NUMBER_OF_ATTRIBUTES];

static int color_setup(void);
static void nocolor_setup(void);

/*
 * Initialize colors. Must only be called once.
 */
void init_colors(void)
{
    /* make sure we only get called once */
#ifndef NDEBUG
    bool first = TRUE;
    assert(first);
    first = FALSE;
#endif /* NDEBUG */

    /* If the `OPT_SKIP_COLORS' option is true
       and the terminal supports colors, initialize colors. */
    if (!is_boolean_option_true(OPT_SKIP_COLORS) && has_colors()) {
	if (color_setup() == -1) {
	    endwin();
	    printf("\nError initializing colors.\n");
	    exit(EXIT_FAILURE);
	}
    } else {
	nocolor_setup(); /* always succeeds */
    }
}

/*
 * Setup for color display. Return 0 on success, -1 on error.
 * Must only be called once.
 */
static int color_setup(void)
{
    int i;
    /* make sure we only get called once */
#ifndef NDEBUG
    bool first = TRUE;
    assert(first);
    first = FALSE;
#endif /* NDEBUG */

    if (start_color() == ERR)
	return -1;

    /* Initialize color pairs */
    for (i = 0; i < NUMBER_OF_ATTRIBUTES; i++)
	if (init_pair(i + 1, color_table[i][0], color_table[i][1]) == ERR)
	    return -1;

#define COLOR_ATTR(highlight, attr_number) \
    ((highlight == TRUE ? A_BOLD : 0) | COLOR_PAIR((attr_number)))

    /* Setup color attributes */
    for (i = 0; i < NUMBER_OF_ATTRIBUTES; i++)
	ColorAttributes[i] = COLOR_ATTR(color_table[i][2], i + 1);

    return 0;
}

/*
 * Setup display for no colors.
 * Must only be called once.
 */
static void nocolor_setup(void)
{
    int i;
    /* make sure we only get called once */
#ifndef NDEBUG
    bool first = TRUE;
    assert(first);
    first = FALSE;
#endif /* NDEBUG */

    for (i = 0; i < NUMBER_OF_ATTRIBUTES; i++)
	ColorAttributes[i] = color_table[i][3];
}
